.popover-content {
  font-weight: normal;
}

td form button.close {
  float: none;
}

.twitter-typeahead .token-input.tt-input {
  height: 20px;
}

.hidden {
  display: none;
}

.input-group-btn .btn {
  height: 100%;
  -moz-box-sizing: content-box;
  box-sizing: content-box;
}

td.filterable a {
  color: #333;
}
